#ifndef ClientThread_Cpp
#define ClientThread_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN

#include <Windows.H>
#include <WinSock2.H>
#include <WinSock.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <IOStream>

#include "SockServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HANDLE ghEvent_ClientThread = NULL;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SocketServer::Start_ClientThread(int iClient)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> SocketServer::Start_ClientThread.\n");
    #endif

    ghEvent_ClientThread = CreateEvent(NULL, FALSE, FALSE, "Client_Thread_Function");

	memset(&CTI, 0, sizeof(CTI));
    CTI.Client = iClient;

    if((hcClient_Thread_Handle[iClient] = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)Client_Thread_Function, (LPVOID) this, 0, (LPDWORD) &dwcClient_Thread_ID[iClient])) == NULL)
    {
        FatalError(__FILE__, __LINE__, "CreateThread failed.");
        CloseHandle(ghEvent_ClientThread);
        return false;
    }

    if(WaitForSingleObject(ghEvent_ClientThread, 5000) == WAIT_TIMEOUT)
    {
        CloseHandle(ghEvent_ClientThread);
        return false;
    }

    CloseHandle(ghEvent_ClientThread);
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SocketServer::WaitOnClientThreadToExit(int iClient)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> SocketServer::WaitOnClientThreadToExit.\n");
    #endif

    DWORD dwExitCode = 0;

    int iWaitCount = 0;

    while(GetExitCodeThread(hcClient_Thread_Handle[iClient], &dwExitCode))
    {
        if(dwExitCode != STILL_ACTIVE || iWaitCount == 500)
        {
            if(iWaitCount == 500)
            {
                printf("Possible thread dead-lock. Client[%d]'s thread will be terminated.\n", iClient);
                TerminateThread(hcClient_Thread_Handle[iClient], 1);
            }

            CloseHandle(hcClient_Thread_Handle[iClient]);
            dwcClient_Thread_ID[iClient] = 0;
            return true;
        }

        iWaitCount++;
        Sleep(1);
    }

    FatalError(__FILE__, __LINE__, "GetExitCodeThread failed.");
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI Client_Thread_Function(LPVOID pvThread)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> Client_Thread_Function.\n");
    #endif

    //---------------------(Initialize Thread [Begin])----------
   	SocketServer *pSockSrvr = ((SocketServer *)pvThread);

	int iClient = ((SocketServer *)pvThread)->CTI.Client;

	SetEvent(ghEvent_ClientThread);
    //---------------------(Initialize Thread [END])----------

    char sRecvBuf[MAXRECVSIZE + 1];
    int iRecvBufSz = 0;

    if(pSockSrvr->IsListeningServer)
    {
        char Status[255];
        sprintf(Status, "Data to client: %d", iClient);
        pSockSrvr->SetNextSendData(iClient, Status);
    }

    while(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
    {
        if(pSockSrvr->IsListeningServer)
        {
			if(pSockSrvr->GetClientData(iClient, sRecvBuf, &iRecvBufSz))
			{
				sRecvBuf[iRecvBufSz] = '\0';

				int iClientLoop = 0;
				while(iClientLoop < pSockSrvr->icMaxClients)
				{
					if(pSockSrvr->bcConnected[iClientLoop] == true && iClientLoop != iClient)
					{
						pSockSrvr->SetNextSendDataEx(iClientLoop, sRecvBuf, iRecvBufSz);
					}
					iClientLoop++;
				}
			}
        }
        else{
			//if(iClient != 0)
			//{
            if(pSockSrvr->GetClientData(iClient, sRecvBuf, &iRecvBufSz))
            {
                sRecvBuf[iRecvBufSz] = '\0';
				//printf("GotData[%d]:%s\n", iClient, sRecvBuf);
				printf("%s\n", sRecvBuf);
            }
			}
			//}

        Sleep(CLIENTTHREAD_MS_WAIT);
    }

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

